// Author: Hezhk3

// see https://gl-transitions.com/editor/InvertedPageCurl

varying highp vec2 textureCoordinate;
varying highp vec2 textureCoordinate2;

uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture2;

uniform highp float iTime;
uniform highp vec2 iResolution;

#define progress fract(iTime)
#define getFromColor(coord) texture2D(inputImageTexture, coord)
#define getToColor(coord) texture2D(inputImageTexture2, coord)
#define ratio (iResolution.x / iResolution.y)

// author: Hewlett-Packard
// license: BSD 3 Clause
// Adapted by Sergey Kosarevsky from:
// http://rectalogic.github.io/webvfx/examples_2transition-shader-pagecurl_8html-example.html

/*
 Copyright (c) 2010 Hewlett-Packard Development Company, L.P. All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
 met:
 
 * Redistributions of source code must retain the above copyright
 notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above
 copyright notice, this list of conditions and the following disclaimer
 in the documentation and/or other materials provided with the
 distribution.
 * Neither the name of Hewlett-Packard nor the names of its
 contributors may be used to endorse or promote products derived from
 this software without specific prior written permission.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 in vec2 texCoord;
 */

const highp float MIN_AMOUNT = -0.16;
const highp float MAX_AMOUNT = 1.5;
#define amount (progress * (MAX_AMOUNT - MIN_AMOUNT) + MIN_AMOUNT)

const highp float PI = 3.141592653589793;

const highp float scale = 512.0;
const highp float sharpness = 3.0;

#define cylinderCenter amount
// 360 degrees * amount
#define cylinderAngle (2.0 * PI * amount)

const highp float cylinderRadius = 1.0 / PI / 2.0;

highp vec3 hitPoint(highp float hitAngle, highp float yc, highp vec3 point, highp mat3 rrotation)
{
    highp float hitPoint = hitAngle / (2.0 * PI);
    point.y = hitPoint;
    return rrotation * point;
}

highp vec4 antiAlias(highp vec4 color1, highp vec4 color2, highp float distanc)
{
    distanc *= scale;
    if (distanc < 0.0) return color2;
    if (distanc > 2.0) return color1;
    highp float dd = pow(1.0 - distanc / 2.0, sharpness);
    return ((color2 - color1) * dd) + color1;
}

highp float distanceToEdge(highp vec3 point)
{
    highp float dx = abs(point.x > 0.5 ? 1.0 - point.x : point.x);
    highp float dy = abs(point.y > 0.5 ? 1.0 - point.y : point.y);
    if (point.x < 0.0) dx = -point.x;
    if (point.x > 1.0) dx = point.x - 1.0;
    if (point.y < 0.0) dy = -point.y;
    if (point.y > 1.0) dy = point.y - 1.0;
    if ((point.x < 0.0 || point.x > 1.0) && (point.y < 0.0 || point.y > 1.0)) return sqrt(dx * dx + dy * dy);
    return min(dx, dy);
}

highp vec4 seeThrough(highp float yc, highp vec2 p, highp mat3 rotation, highp mat3 rrotation)
{
    highp float hitAngle = PI - (acos(yc / cylinderRadius) - cylinderAngle);
    highp vec3 point = hitPoint(hitAngle, yc, rotation * vec3(p, 1.0), rrotation);
    if (yc <= 0.0 && (point.x < 0.0 || point.y < 0.0 || point.x > 1.0 || point.y > 1.0))
    {
        return getToColor(p);
    }
    
    if (yc > 0.0) return getFromColor(p);
    
    highp vec4 color = getFromColor(point.xy);
    highp vec4 tcolor = vec4(0.0);
    
    return antiAlias(color, tcolor, distanceToEdge(point));
}

highp vec4 seeThroughWithShadow(highp float yc, highp vec2 p, highp vec3 point, highp mat3 rotation, highp mat3 rrotation)
{
    highp float shadow = distanceToEdge(point) * 30.0;
    shadow = (1.0 - shadow) / 3.0;
    
    if (shadow < 0.0) shadow = 0.0; else shadow *= amount;
    
    highp vec4 shadowColor = seeThrough(yc, p, rotation, rrotation);
    shadowColor.r -= shadow;
    shadowColor.g -= shadow;
    shadowColor.b -= shadow;
    
    return shadowColor;
}

highp vec4 backside(highp float yc, highp vec3 point)
{
    highp vec4 color = getFromColor(point.xy);
    highp float gray = (color.r + color.b + color.g) / 15.0;
    gray += (8.0 / 10.0) * (pow(1.0 - abs(yc / cylinderRadius), 2.0 / 10.0) / 2.0 + (5.0 / 10.0));
    color.rgb = vec3(gray);
    return color;
}

highp vec4 behindSurface(highp vec2 p, highp float yc, highp vec3 point, highp mat3 rrotation)
{
    highp float shado = (1.0 - ((-cylinderRadius - yc) / amount * 7.0)) / 6.0;
    shado *= 1.0 - abs(point.x - 0.5);
    
    yc = (-cylinderRadius - cylinderRadius - yc);
    
    highp float hitAngle = (acos(yc / cylinderRadius) + cylinderAngle) - PI;
    point = hitPoint(hitAngle, yc, point, rrotation);
    
    if (yc < 0.0 && point.x >= 0.0 && point.y >= 0.0 && point.x <= 1.0 && point.y <= 1.0 && (hitAngle < PI || amount > 0.5))
    {
        shado = 1.0 - (sqrt(pow(point.x - 0.5, 2.0) + pow(point.y - 0.5, 2.0)) / (71.0 / 100.0));
        shado *= pow(-yc / cylinderRadius, 3.0);
        shado *= 0.5;
    }
    else
    {
        shado = 0.0;
    }
    return vec4(getToColor(p).rgb - shado, 1.0);
}

highp vec4 transition(highp vec2 p) {
    
    const highp float angle = 100.0 * PI / 180.0;
    highp float c = cos(-angle);
    highp float s = sin(-angle);
    
    highp mat3 rotation = mat3( c, s, 0,
                               -s, c, 0,
                               -0.801, 0.8900, 1
                               );
    c = cos(angle);
    s = sin(angle);
    
    highp mat3 rrotation = mat3(    c, s, 0,
                                -s, c, 0,
                                0.98500, 0.985, 1
                                );
    
    highp vec3 point = rotation * vec3(p, 1.0);
    
    highp float yc = point.y - cylinderCenter;
    
    if (yc < -cylinderRadius)
    {
        // Behind surface
        return behindSurface(p,yc, point, rrotation);
    }
    
    if (yc > cylinderRadius)
    {
        // Flat surface
        return getFromColor(p);
    }
    
    highp float hitAngle = (acos(yc / cylinderRadius) + cylinderAngle) - PI;
    
    highp float hitAngleMod = mod(hitAngle, 2.0 * PI);
    if ((hitAngleMod > PI && amount < 0.5) || (hitAngleMod > PI/2.0 && amount < 0.0))
    {
        return seeThrough(yc, p, rotation, rrotation);
    }
    
    point = hitPoint(hitAngle, yc, point, rrotation);
    
    if (point.x < 0.0 || point.y < 0.0 || point.x > 1.0 || point.y > 1.0)
    {
        return seeThroughWithShadow(yc, p, point, rotation, rrotation);
    }
    
    highp vec4 color = backside(yc, point);
    
    highp vec4 otherColor;
    if (yc < 0.0)
    {
        highp float shado = 1.0 - (sqrt(pow(point.x - 0.5, 2.0) + pow(point.y - 0.5, 2.0)) / 0.71);
        shado *= pow(-yc / cylinderRadius, 3.0);
        shado *= 0.5;
        otherColor = vec4(0.0, 0.0, 0.0, shado);
    }
    else
    {
        otherColor = getFromColor(p);
    }
    
    color = antiAlias(color, otherColor, cylinderRadius - abs(yc));
    
    highp vec4 cl = seeThroughWithShadow(yc, p, point, rotation, rrotation);
    highp float dist = distanceToEdge(point);
    
    return antiAlias(color, cl, dist);
}

void main()
{
    gl_FragColor = transition(textureCoordinate);
}
