// Author: Hezhk3

// see https://gl-transitions.com/editor/StereoViewer

varying highp vec2 textureCoordinate;
varying highp vec2 textureCoordinate2;

uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture2;

uniform highp float iTime;
uniform highp vec2 iResolution;

#define progress fract(iTime)
#define getFromColor(coord) texture2D(inputImageTexture, coord)
#define getToColor(coord) texture2D(inputImageTexture2, coord)
//#define ratio (iResolution.x / iResolution.y)

// Tunable parameters
// How much to zoom (out) for the effect ~ 0.5 - 1.0
const highp float zoom = 0.88;
                    // Corner radius as a fraction of the image height
const highp float corner_radius = 0.22;

// author: Ted Schundler
// license: BSD 2 Clause
// Free for use and modification by anyone with credit

// Copyright (c) 2016, Theodore K Schundler
// All rights reserved.

// Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

// 1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.

// 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.

// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

///////////////////////////////////////////////////////////////////////////////
// Stereo Viewer Toy Transition                                              //
//                                                                           //
// Inspired by ViewMaster / Image3D image viewer devices.                    //
// This effect is similar to what you see when you press the device's lever. //
// There is a quick zoom in / out to make the transition 'valid' for GLSL.io //
///////////////////////////////////////////////////////////////////////////////

const highp vec4 black = vec4(0.0, 0.0, 0.0, 1.0);
const highp vec2 c00 = vec2(0.0, 0.0); // the four corner points
const highp vec2 c01 = vec2(0.0, 1.0);
const highp vec2 c11 = vec2(1.0, 1.0);
const highp vec2 c10 = vec2(1.0, 0.0);

// Check if a point is within a given corner
bool in_corner(highp vec2 p, highp vec2 corner, highp vec2 radius) {
    // determine the direction we want to be filled
    highp vec2 axis = (c11 - corner) - corner;
    
    // warp the point so we are always testing the bottom left point with the
    // circle centered on the origin
    p = p - (corner + axis * radius);
    p *= axis / radius;
    return (p.x > 0.0 && p.y > -1.0) || (p.y > 0.0 && p.x > -1.0) || dot(p, p) < 1.0;
}

// Check all four corners
// return a float for v2 for anti-aliasing?
bool test_rounded_mask(highp vec2 p, highp vec2 corner_size) {
    return
    in_corner(p, c00, corner_size) &&
    in_corner(p, c01, corner_size) &&
    in_corner(p, c10, corner_size) &&
    in_corner(p, c11, corner_size);
}

// Screen blend mode - https://en.wikipedia.org/wiki/Blend_modes
// This more closely approximates what you see than linear blending
highp vec4 screen(highp vec4 a, highp vec4 b) {
    return 1.0 - (1.0 - a) * (1.0 -b);
}

// Given RGBA, find a value that when screened with itself
// will yield the original value.
highp vec4 unscreen(highp vec4 c) {
    return 1.0 - sqrt(1.0 - c);
}

// Grab a pixel, only if it isn't masked out by the rounded corners
highp vec4 sample_with_corners_from(highp vec2 p, highp vec2 corner_size) {
    p = (p - 0.5) / zoom + 0.5;
    if (!test_rounded_mask(p, corner_size)) {
        return black;
    }
    return unscreen(getFromColor(p));
}

highp vec4 sample_with_corners_to(highp vec2 p, highp vec2 corner_size) {
    p = (p - 0.5) / zoom + 0.5;
    if (!test_rounded_mask(p, corner_size)) {
        return black;
    }
    return unscreen(getToColor(p));
}

// special sampling used when zooming - extra zoom parameter and don't unscreen
highp vec4 simple_sample_with_corners_from(highp vec2 p, highp vec2 corner_size, highp float zoom_amt) {
    p = (p - 0.5) / (1.0 - zoom_amt + zoom * zoom_amt) + 0.5;
    if (!test_rounded_mask(p, corner_size)) {
        return black;
    }
    return getFromColor(p);
}

highp vec4 simple_sample_with_corners_to(highp vec2 p, highp vec2 corner_size, highp float zoom_amt) {
    p = (p - 0.5) / (1.0 - zoom_amt + zoom * zoom_amt) + 0.5;
    if (!test_rounded_mask(p, corner_size)) {
        return black;
    }
    return getToColor(p);
}

// Basic 2D affine transform matrix helpers
// These really shouldn't be used in a fragment shader - I should work out the
// the math for a translate & rotate function as a pair of dot products instead

highp mat3 rotate2d(highp float angle, highp float ratio) {
    highp float s = sin(angle);
    highp float c = cos(angle);
    return mat3(
                c, s ,0.0,
                -s, c, 0.0,
                0.0, 0.0, 1.0);
}

highp mat3 translate2d(highp float x, highp float y) {
    return mat3(
                1.0, 0.0, 0,
                0.0, 1.0, 0,
                -x, -y, 1.0);
}

highp mat3 scale2d(highp float x, highp float y) {
    return mat3(
                x, 0.0, 0,
                0.0, y, 0,
                0, 0, 1.0);
}

// Split an image and rotate one up and one down along off screen pivot points
highp vec4 get_cross_rotated(highp vec3 p3, highp float angle, highp vec2 corner_size, highp float ratio) {
    angle = angle * angle; // easing
    angle /= 2.4; // works out to be a good number of radians
    
    highp mat3 center_and_scale = translate2d(-0.5, -0.5) * scale2d(1.0, ratio);
    highp mat3 unscale_and_uncenter = scale2d(1.0, 1.0/ratio) * translate2d(0.5,0.5);
    highp mat3 slide_left = translate2d(-2.0,0.0);
    highp mat3 slide_right = translate2d(2.0,0.0);
    highp mat3 rotate = rotate2d(angle, ratio);
    
    highp mat3 op_a = center_and_scale * slide_right * rotate * slide_left * unscale_and_uncenter;
    highp mat3 op_b = center_and_scale * slide_left * rotate * slide_right * unscale_and_uncenter;
    
    highp vec4 a = sample_with_corners_from((op_a * p3).xy, corner_size);
    highp vec4 b = sample_with_corners_from((op_b * p3).xy, corner_size);
    
    return screen(a, b);
}

// Image stays put, but this time move two masks
highp vec4 get_cross_masked(highp vec3 p3, highp float angle, highp vec2 corner_size, highp float ratio) {
    angle = 1.0 - angle;
    angle = angle * angle; // easing
    angle /= 2.4;
    
    highp vec4 img;
    
    highp mat3 center_and_scale = translate2d(-0.5, -0.5) * scale2d(1.0, ratio);
    highp mat3 unscale_and_uncenter = scale2d(1.0 / zoom, 1.0 / (zoom * ratio)) * translate2d(0.5,0.5);
    highp mat3 slide_left = translate2d(-2.0,0.0);
    highp mat3 slide_right = translate2d(2.0,0.0);
    highp mat3 rotate = rotate2d(angle, ratio);
    
    highp mat3 op_a = center_and_scale * slide_right * rotate * slide_left * unscale_and_uncenter;
    highp mat3 op_b = center_and_scale * slide_left * rotate * slide_right * unscale_and_uncenter;
    
    bool mask_a = test_rounded_mask((op_a * p3).xy, corner_size);
    bool mask_b = test_rounded_mask((op_b * p3).xy, corner_size);
    
    if (mask_a || mask_b) {
        img = sample_with_corners_to(p3.xy, corner_size);
        return screen(mask_a ? img : black, mask_b ? img : black);
    } else {
        return black;
    }
}

highp vec4 transition(highp vec2 uv) {
    highp float ratio = iResolution.x / iResolution.y;

    highp float a;
    highp vec2 p=uv.xy/vec2(1.0).xy;
    highp vec3 p3 = vec3(p.xy, 1.0); // for 2D matrix transforms
    
    // corner is warped to represent to size after mapping to 1.0, 1.0
    highp vec2 corner_size = vec2(corner_radius / ratio, corner_radius);
    
    if (progress <= 0.0) {
        // 0.0: start with the base frame always
        return getFromColor(p);
    } else if (progress < 0.1) {
        // 0.0-0.1: zoom out and add rounded corners
        a = progress / 0.1;
        return  simple_sample_with_corners_from(p, corner_size * a, a);
    } else if (progress < 0.48) {
        // 0.1-0.48: Split original image apart
        a = (progress - 0.1)/0.38;
        return get_cross_rotated(p3, a, corner_size, ratio);
    } else if (progress < 0.9) {
        // 0.48-0.52: black
        // 0.52 - 0.9: unmask new image
        return get_cross_masked(p3, (progress - 0.52)/0.38, corner_size, ratio);
    } else if (progress < 1.0) {
        // zoom out and add rounded corners
        a = (1.0 - progress) / 0.1;
        return simple_sample_with_corners_to(p, corner_size * a, a);
    } else {
        // 1.0 end with base frame
        return getToColor(p);
    }
}


void main()
{
    gl_FragColor = transition(textureCoordinate);
}
